/*
 * Copyright (c) 2008-2013 Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global NX, Ext, Sonatype, Nexus*/
NX.define('Nexus.ssl.CertificateStore', {
        extend : 'Ext.data.Store',
        requireSuper : false,
        singleton : true,

        constructor : function() {
          var self = this, url = Sonatype.config.contextPath + '/service/siesta/ssl/truststore';

          self.autoSave = true;
          self.restful = true;

          self.reader = new Ext.data.JsonReader({
            idProperty : 'id',
            fields : ['id',
              'subjectCommonName', 'subjectOrganization', 'subjectOrganizationalUnit',
              'issuerCommonName', 'issuerOrganization', 'issuerOrganizationalUnit',
              'fingerprint', 'pem', 'serialNumber',
              'issuedOn', 'expiresOn'
            ]
          });

          self.writer = new Ext.data.JsonWriter({
            encode : false,
            writeAllFields : true,

            /**
             * Default JsonWriter is forcing root object 'trustedKeys', we need the data unwrapped.
             */
            render : function(params, baseParams, data) {
              Ext.applyIf(data, params, baseParams);
              params.jsonData = data;
            }
          });

          self.proxy = new Ext.data.HttpProxy({
            api : {
              read : {
                url : url,
                method : 'GET'
              },
              create : {
                url : url,
                method : 'POST'
              },
              update : {
                url : url,
                method : 'PUT'
              },
              destroy : {
                url : url,
                method : 'DELETE'
              }
            }
          });

          self.listeners = {
            beforewrite : function(store, action, rs) {
              // HACK/EXTJSBUG dirty flag would show up in grid although it's autosaved
              rs.dirty = false;
            }
          };

          self.constructor.superclass.constructor.apply(self, arguments);
        }
      }
);