/*
 * Copyright (c) 2008-2013 Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global NX, Ext, Nexus*/

/**
 * Container for icons used by staging.
 *
 * @since 2.3
 */
NX.define('Nexus.staging.Icons', {
    extend: 'Nexus.util.IconContainer',
    singleton: true,

    /**
     * @constructor
     */
    constructor: function() {
        var self = this;

        // helper to build an icon config with variants, where variants live in directories, foo.png x16 -> x16/foo.png
        function iconConfig(fileName, variants) {
            var config = {};
            if (variants === undefined) {
                variants = [ 'x32', 'x16' ]
            }
            Ext.each(variants, function(variant) {
                config[variant] = variant + '/' + fileName;
            });
            return config;
        }

        self.constructor.superclass.constructor.call(self, {
            stylePrefix: 'nx-staging-icons-',

            icons: {
                accept:             iconConfig('accept.png'),
                bug:                iconConfig('bug.png'),
                cog:                iconConfig('cog.png'),
                cog_green:          iconConfig('cog_green.png'),
                cog_red:            iconConfig('cog_red.png'),
                cross:              iconConfig('cross.png'),
                database:           iconConfig('database.png'),
                database_back:      iconConfig('database_back.png'),
                database_delete:    iconConfig('database_delete.png'),
                database_edit:      iconConfig('database_edit.png'),
                database_go:        iconConfig('database_go.png'),
                database_link:      iconConfig('database_link.png'),
                database_save:      iconConfig('database_save.png'),
                email:              iconConfig('email.png'),
                error:              iconConfig('error.png'),
                gear:               '@cog',
                gear_in:            iconConfig('gear_in.png'),
                gear_in_green:      iconConfig('gear_in_green.png'),
                gear_in_red:        iconConfig('gear_in_red.png'),
                page_copy:          iconConfig('page_copy.png'),
                exclamation:        iconConfig('exclamation.png'),
                folder_database:    iconConfig('folder_database.png'),
                folder_go:          iconConfig('folder_go.png'),
                star:               iconConfig('star.png'),
                tick:               iconConfig('tick.png'),
                warning:            '@error',

                // common
                arrow_refresh:      'arrow_refresh.png', // use silk version here w/o any variants to keep refresh consistent across the app
                refresh:            '@arrow_refresh',
                selectionMulti:     'selection-select.png', // this is from fugue icon set
                selectionEmpty:     '@warning',

                // repository types
                repositoryOpened:           '@database_edit',
                repositoryClosed:           '@database',
                repositoryPromoted:         '@database_go',
                repositoryGroup:            '@folder_database',
                repositoryGroupPromoted:    '@folder_go',
                repositoryGroupReleased:    '@database_link',
                repositoryReleased:         '@database_link',

                // repository actions
                closeRepository:            '@database_save',
                dropRepository:             '@database_delete',
                releaseRepository:          '@database_link',
                promoteRepository:          '@database_go',

                transitioning:              '@cog',
                transitioningBadge:         'gear-small.png',

                // activities
                activity_open:              '@database_edit',
                activity_close:             '@database_save',
                activity_promote:           '@database_go',
                activity_group:             '@folder_database',
                activity_release:           '@database_link',
                activity_drop:              '@database_delete',
                activity_demote:            '@database_back',

                // events
                eventSuccess:                   '@accept',
                eventFailure:                   '@exclamation',
                event_error:                    '@error',
                event_repositoryCreated:        '@eventSuccess',
                event_repositoryGroupCreated:   '@eventSuccess',
                event_repositoryClosed:         '@eventSuccess',
                event_repositoryCloseFailed:    '@eventFailure',
                event_repositoryPromoted:       '@eventSuccess',
                event_repositoryPromoteFailed:  '@eventFailure',
                event_repositoryDemoted:        '@eventSuccess',
                event_repositoryDemoteFailed:   '@eventFailure',
                event_repositoryReleased:       '@eventSuccess',
                event_repositoryReleaseFailed:  '@eventFailure',

                // rules
                event_rulesEvaluate:            '@gear_in',
                event_rulesPassed:              '@gear_in_green',
                event_rulesFailed:              '@gear_in_red',
                event_ruleEvaluate:             '@cog',
                event_rulePassed:               '@cog_green',
                event_ruleFailed:               '@cog_red',

                // email
                event_email:                    '@email',

                // copy items
                event_copyItems:                '@page_copy',

                // notifications
                notification_0:     '@accept',
                notification_1:     'notification-counter.png',
                notification_2:     'notification-counter-02.png',
                notification_3:     'notification-counter-03.png',
                notification_4:     'notification-counter-04.png',
                notification_5:     'notification-counter-05.png',
                notification_6:     'notification-counter-06.png',
                notification_7:     'notification-counter-07.png',
                notification_8:     'notification-counter-08.png',
                notification_9:     'notification-counter-09.png',
                notification_10:    'notification-counter-10.png',
                notification_11:    'notification-counter-11.png',
                notification_12:    'notification-counter-12.png',
                notification_13:    'notification-counter-13.png',
                notification_14:    'notification-counter-14.png',
                notification_15:    'notification-counter-15.png',
                notification_16:    'notification-counter-16.png',
                notification_17:    'notification-counter-17.png',
                notification_18:    'notification-counter-18.png',
                notification_19:    'notification-counter-19.png',
                notification_20:    'notification-counter-20.png',
                notification_more:  'notification-counter-20-plus.png'
            }
        });
    },

    //
    // FIXME: Should add this feature to the IconContainer, so that instead of exceptions we'd instead
    // FIXME: get some default icon which might help show problems visually but without causing javascript errors.
    //
    // FIXME: Since get() still throws an exception and its used all over, even use of invalidIcon() here
    // FIXME: could still cause javascript errors.
    //

    /**
     * Return icon to use when otherwise would be invalid.
     *
     * @private
     */
    invalidIcon: function(message) {
        this.logWarn(message);
        return this.get('bug');
    },

    /**
     * Determine icon for a given repository record data.
     *
     * @return {*} Icon; never null/undefined.
     */
    forRepository: function(repo) {
        var self = this;

        // promotion groups have a bit more work to figure out which icon to use
        if (repo.profileType === 'group') {
            if (repo.type === 'promoted') {
                return self.get('repositoryGroupPromoted');
            }
            else if (repo.type === 'released') {
                return self.get('repositoryGroupReleased');
            }
            else {
                return self.get('repositoryGroup');
            }
        }

        // else staging repository, has simple state -> icon
        if (repo.type === 'open') {
            return self.get('repositoryOpened');
        }
        else if (repo.type === 'closed') {
            return self.get('repositoryClosed');
        }
        else if (repo.type === 'promoted') {
            return self.get('repositoryPromoted');
        }
        else if (repo.type === 'released') {
            return self.get('repositoryReleased');
        }

        return self.invalidIcon('Unable to determine icon for: ' + repo);
    },

    /**
     * Determine icon for notification count.
     *
     * @return {*} Icon; never null/undefined.
     */
    forNotificationCount: function (count) {
        NX.assert(Ext.isNumber(count) && count >= 0);

        if (count > 20) {
            return this.get('notification_more');
        }
        else {
            return this.get('notification_' + count);
        }
    },

    // TODO: Probably should look into helpers to simplify making/using badges

    /**
     * Determine HTML for a repository icon with an optional badge.
     *
     * @return {String} Icon + optional badge HTML.
     */
    renderRepositoryWithBadge: function(repo) {
        var iconImg = this.forRepository(repo).img,
            badgeCls;

        if (repo.transitioning) {
            badgeCls = this.get('transitioningBadge').cls;
            return '<div><span class="' + badgeCls + ' nx-staging-icons-transitioning-badge"></span>' + iconImg + '</div>';
        }
        else if (repo.notifications > 0) {
            badgeCls = this.forNotificationCount(repo.notifications).cls;
            return '<div><span class="' + badgeCls + ' nx-staging-icons-notification-badge"></span>' + iconImg + '</div>';
        }
        else {
            return iconImg;
        }
    },

    /**
     * Determine icon for a given activity name.
     *
     * @param name  Activity name.
     * @return {*}  Icon; never null/undefined.
     */
    forActivityName: function(name) {
        return this.get('activity_' + name);
    },

    /**
     * Determine icon for a given event name.
     *
     *  @return {*} Icon; never null/undefined.
     */
    forEventName: function(name) {
        return this.get('event_' + name);
    }
});