/*
 * Copyright (c) 2008-2013 Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global NX,Ext,Sonatype,Nexus*/
NX.define('Sonatype.repoServer.LdapConfigurationUserGroup', {
  extend : 'Ext.Panel',
  requires : [
    'Sonatype.repoServer.LdapUserMappingPanel',
    'Sonatype.repoServer.LdapLoginTestWindow'
  ],
  constructor : function(cfg) {
    var config = cfg || {};
    var defaultConfig = {
      fieldWidth : 200
    };
    Ext.apply(this, config, defaultConfig);

    var groupTypeStore = new Ext.data.SimpleStore({
      fields : ['value', 'display'],
      data : [
        ['dynamic', 'Dynamic Groups'],
        ['static', 'Static Groups']
      ]
    });

    var templateStore = new Ext.data.JsonStore({
      root : 'data',
      id : 'name',
      fields : [
        {
          name : 'name',
          sortType : Ext.data.SortTypes.asUCString
        },
        {
          name : 'userAndGroupConfig'
        }
      ],
      url : Sonatype.config.servicePath + '/ldap/templates',
      autoLoad : true,
      sortInfo : {
        field : 'name',
        direction : 'asc'
      }
    });

    Sonatype.repoServer.LdapConfigurationUserGroup.superclass.constructor.call(this, {
      items : [
        {
          xtype : 'fieldset',
          checkboxToggle : false,
          title : 'User & Group Templates',
          labelWidth : 188,
          collapsible : true,
          collapsed : false,
          autoHeight : true,
          items : [
            {
              xtype : 'combo',
              fieldLabel : 'Template',
              helpText : 'Select a template to apply some default values to the configuration.',
              name : 'template',
              width : this.fieldWidth,
              store : templateStore,
              valueField : 'name',
              displayField : 'name',
              editable : false,
              forceSelection : true,
              mode : 'local',
              triggerAction : 'all',
              emptyText : 'Select a Template to apply...',
              selectOnFocus : true,
              listeners : {
                select : {
                  fn : this.templateSelectHandler,
                  scope : this
                }
              }
            }
          ]
        },
        {
          xtype : 'fieldset',
          checkboxToggle : false,
          title : 'User Element Mapping',
          labelWidth : 188,
          collapsible : true,
          autoHeight : true,
          items : [
            {
              xtype : 'textfield',
              fieldLabel : 'Base DN',
              helpText : 'Base location in the LDAP that the users are found, relative to the search base ("ou=people").',
              name : 'userAndGroupConfig.userBaseDn',
              width : this.fieldWidth,
              allowBlank : true
            },
            {
              xtype : 'checkbox',
              fieldLabel : 'User Subtree',
              helpText : 'Check this box if users are located in structures below the '
                    + 'user Base DN. If all users are located in ou=people,dc=example,dc=com, '
                    + 'this value should be false. If users are located in organizations such '
                    + 'as ou=development,ou=people,dc=example,dc=com, this value should be true.',
              name : 'userAndGroupConfig.userSubtree'
            },
            {
              xtype : 'textfield',
              fieldLabel : 'Object Class',
              helpText : 'LDAP class for user objects ("inetOrgPerson").',
              name : 'userAndGroupConfig.userObjectClass',
              width : this.fieldWidth,
              itemCls : 'required-field',
              allowBlank : false
            },
            {
              xtype : 'textfield',
              fieldLabel : 'User Filter',
              helpText : 'LDAP search filter to additionally limit user search (for example "attribute=foo" or "(|(mail=*@domain.com)(uid=dom*))".',
              name : 'userAndGroupConfig.ldapFilter',
              width : this.fieldWidth,
              allowBlank : true
            },
            {
              xtype : 'textfield',
              fieldLabel : 'User ID Attribute',
              helpText : 'LDAP attribute containing user id ("userIdAttribute").',
              name : 'userAndGroupConfig.userIdAttribute',
              width : this.fieldWidth,
              itemCls : 'required-field',
              allowBlank : false
            },
            {
              xtype : 'textfield',
              fieldLabel : 'Real Name Attribute',
              helpText : 'LDAP attribute containing the real name of the user ("cn").',
              name : 'userAndGroupConfig.userRealNameAttribute',
              width : this.fieldWidth,
              itemCls : 'required-field',
              allowBlank : false
            },
            {
              xtype : 'textfield',
              fieldLabel : 'E-Mail Attribute',
              helpText : 'LDAP attribute containing e-mail address ("emailAddressAttribute").',
              name : 'userAndGroupConfig.emailAddressAttribute',
              width : this.fieldWidth,
              itemCls : 'required-field',
              allowBlank : false
            }
          ]
        },
        {
          xtype : 'fieldset',
          checkboxToggle : true,
          labelWidth : 188,
          collapsed : true,
          title : 'Use Password Attribute',
          autoHeight : true,
          listeners : {
            'expand' : {
              fn : function(panel) {
                this.ldapConfigurationContainer.expandHandler(panel);
              },
              scope : this
            },
            'collapse' : {
              fn : function(panel) {
                this.find('name', 'userAndGroupConfig.userPasswordAttribute')[0].setValue(null);
                this.ldapConfigurationContainer.collapseHandler(panel);
              },
              scope : this,
              delay : 100
            }
          },
          items : [
            {
              xtype : 'textfield',
              fieldLabel : 'Password Attribute',
              helpText : 'LDAP attribute containing the password ("userPassword").  If this field is blank the user will be authenticated against a bind with the LDAP server.',
              name : 'userAndGroupConfig.userPasswordAttribute',
              width : this.fieldWidth,
              allowBlank : false,
              itemCls : 'required-field'
            }
          ]
        },
        {
          xtype : 'fieldset',
          checkboxToggle : true,
          labelWidth : 188,
          collapsed : false,
          title : 'Group Element Mapping',
          autoHeight : true,
          listeners : {
            'expand' : {
              fn : function(panel) {
                this.find('name', 'userAndGroupConfig.ldapGroupsAsRoles')[0].setValue(true);
                this.ldapConfigurationContainer.expandHandler(panel);
              },
              scope : this
            },
            'collapse' : {
              fn : function(panel) {
                this.find('name', 'userAndGroupConfig.ldapGroupsAsRoles')[0].setValue(false);
                this.ldapConfigurationContainer.collapseHandler(panel);
              },
              scope : this,
              delay : 100
            }
          },
          items : [
            {
              xtype : 'combo',
              itemCls : 'required-field',
              fieldLabel : 'Group Type',
              helpText : 'The type of group to use, static or dynamic.',
              name : 'groupType',
              width : this.fieldWidth,
              store : groupTypeStore,
              valueField : 'value',
              displayField : 'display',
              editable : false,
              forceSelection : true,
              mode : 'local',
              triggerAction : 'all',
              emptyText : 'Select...',
              selectOnFocus : true,
              allowBlank : false,
              listeners : {
                select : {
                  fn : this.groupTypeSelectHandler,
                  scope : this
                }
              }
            },
            {
              xtype : 'textfield',
              fieldLabel : 'Base DN',
              helpText : 'Base location in the LDAP that the groups are found, relative to the search base ("ou=Group").',
              name : 'userAndGroupConfig.groupBaseDn',
              width : this.fieldWidth,
              allowBlank : true,
              hideParent : true
            },
            {
              xtype : 'checkbox',
              fieldLabel : 'Group Subtree',
              helpText : 'Check this box if groups are located in structures below the '
                    + 'group Base DN. If all groups are located in ou=group,dc=example,dc=com, '
                    + 'this value should be false. If groups are located in organizations such '
                    + 'as ou=devgroups,ou=groups,dc=example,dc=com, this value should be true.',
              name : 'userAndGroupConfig.groupSubtree',
              hideParent : true
            },
            {
              xtype : 'textfield',
              fieldLabel : 'Object Class',
              helpText : 'LDAP class for group objects ("posixGroup").',
              name : 'userAndGroupConfig.groupObjectClass',
              width : this.fieldWidth,
              itemCls : 'required-field',
              allowBlank : false,
              hideParent : true
            },
            {
              xtype : 'textfield',
              fieldLabel : 'Group ID Attribute',
              helpText : 'LDAP attribute containing group id ("cn").',
              name : 'userAndGroupConfig.groupIdAttribute',
              width : this.fieldWidth,
              itemCls : 'required-field',
              allowBlank : false,
              hideParent : true
            },
            {
              xtype : 'textfield',
              fieldLabel : 'Group Member Attribute',
              helpText : 'LDAP attribute containing the usernames for the group.',
              name : 'userAndGroupConfig.groupMemberAttribute',
              width : this.fieldWidth,
              itemCls : 'required-field',
              allowBlank : false,
              hideParent : true
            },
            {
              xtype : 'textfield',
              fieldLabel : 'Group Member Format',
              helpText : 'The format of User ID stored in the Group Member Attribute. A token "${dn}" can be used to lookup the FQDN of the user or use something like "uid=${username},ou=people,o=sonatype" where "${username}" is replaced with the Username value.',
              name : 'userAndGroupConfig.groupMemberFormat',
              width : this.fieldWidth,
              itemCls : 'required-field',
              allowBlank : false,
              hideParent : true
            },
            {
              xtype : 'textfield',
              fieldLabel : 'Member of Attribute',
              helpText : 'Groups are generally one of two types in LDAP systems -'
                    + ' static or dynamic. A static group maintains its own'
                    + ' membership list. A dynamic group records its membership on a'
                    + ' user entry. If dynamic groups this should be set to the'
                    + ' attribute used to store the attribute that holds groups DN in the user object.',
              name : 'userAndGroupConfig.userMemberOfAttribute',
              width : this.fieldWidth,
              itemCls : 'required-field',
              allowBlank : false,
              hideParent : true
            },
            {
              xtype : 'checkbox',
              hidden : true,
              hideLabel : true,
              height : 0,
              name : 'userAndGroupConfig.ldapGroupsAsRoles',
              checked : true
            }
          ]
        }
      ],
      listeners : {
        afterlayout : {
          fn : function() {
            this.isLoaded = true;
          },
          scope : this
        }
      }
    });
  },

  postLoad : function() {
    if (!this.ldapConfigurationContainer.isNew) {
      this.find('name', 'template')[0].ownerCt.collapse();

      var ldapGroupsAsRoles = this.find('name', 'userAndGroupConfig.ldapGroupsAsRoles')[0];
      if (!ldapGroupsAsRoles.getValue()) {
        ldapGroupsAsRoles.ownerCt.collapse();
      }

      var userPassAttrib = this.find('name', 'userAndGroupConfig.userPasswordAttribute')[0];
      if (Ext.isEmpty(userPassAttrib.getValue())) {
        userPassAttrib.ownerCt.collapse();
      }
      else {
        userPassAttrib.ownerCt.expand();
      }

      if (Ext.isEmpty(this.find('name', 'userAndGroupConfig.userMemberOfAttribute')[0].getValue())) {
        this.find('name', 'groupType')[0].setValue('static');
        this.hideComponent(this.find('name', 'userAndGroupConfig.userMemberOfAttribute')[0]);
        this.showComponent(this.find('name', 'userAndGroupConfig.groupBaseDn')[0]);
        this.showComponent(this.find('name', 'userAndGroupConfig.groupSubtree')[0]);
        this.showComponent(this.find('name', 'userAndGroupConfig.groupObjectClass')[0]);
        this.showComponent(this.find('name', 'userAndGroupConfig.groupIdAttribute')[0]);
        this.showComponent(this.find('name', 'userAndGroupConfig.groupMemberAttribute')[0]);
        this.showComponent(this.find('name', 'userAndGroupConfig.groupMemberFormat')[0]);
      }
      else {
        this.find('name', 'groupType')[0].setValue('dynamic');
        this.showComponent(this.find('name', 'userAndGroupConfig.userMemberOfAttribute')[0]);
        this.hideComponent(this.find('name', 'userAndGroupConfig.groupBaseDn')[0]);
        this.hideComponent(this.find('name', 'userAndGroupConfig.groupSubtree')[0]);
        this.hideComponent(this.find('name', 'userAndGroupConfig.groupObjectClass')[0]);
        this.hideComponent(this.find('name', 'userAndGroupConfig.groupIdAttribute')[0]);
        this.hideComponent(this.find('name', 'userAndGroupConfig.groupMemberAttribute')[0]);
        this.hideComponent(this.find('name', 'userAndGroupConfig.groupMemberFormat')[0]);
      }
    }
  },
  applyFieldValue : function(field, value) {
    field.setValue(value);
    field.clearInvalid();
  },
  groupTypeSelectHandler : function(combo, record, index) {
    var newValue = combo.getValue();

    if (newValue == 'static') {
      // clear out this field
      this.find('name', 'userAndGroupConfig.userMemberOfAttribute')[0].setValue('');
      this.hideComponent(this.find('name', 'userAndGroupConfig.userMemberOfAttribute')[0]);
      this.showComponent(this.find('name', 'userAndGroupConfig.groupBaseDn')[0]);
      this.showComponent(this.find('name', 'userAndGroupConfig.groupSubtree')[0]);
      this.showComponent(this.find('name', 'userAndGroupConfig.groupObjectClass')[0]);
      this.showComponent(this.find('name', 'userAndGroupConfig.groupIdAttribute')[0]);
      this.showComponent(this.find('name', 'userAndGroupConfig.groupMemberAttribute')[0]);
      this.showComponent(this.find('name', 'userAndGroupConfig.groupMemberFormat')[0]);
    }
    else {
      this.showComponent(this.find('name', 'userAndGroupConfig.userMemberOfAttribute')[0]);
      this.hideComponent(this.find('name', 'userAndGroupConfig.groupBaseDn')[0]);
      this.hideComponent(this.find('name', 'userAndGroupConfig.groupSubtree')[0]);
      this.hideComponent(this.find('name', 'userAndGroupConfig.groupObjectClass')[0]);
      this.hideComponent(this.find('name', 'userAndGroupConfig.groupIdAttribute')[0]);
      this.hideComponent(this.find('name', 'userAndGroupConfig.groupMemberAttribute')[0]);
      this.hideComponent(this.find('name', 'userAndGroupConfig.groupMemberFormat')[0]);
    }

    this.doLayout();
  },
  templateSelectHandler : function(combo, record, index) {

    var data = record.data.userAndGroupConfig;

    this.applyFieldValue(this.find('name', 'userAndGroupConfig.userBaseDn')[0], data.userBaseDn);
    this.applyFieldValue(this.find('name', 'userAndGroupConfig.userSubtree')[0], data.userSubtree);
    this.applyFieldValue(this.find('name', 'userAndGroupConfig.userObjectClass')[0], data.userObjectClass);
    this.applyFieldValue(this.find('name', 'userAndGroupConfig.userIdAttribute')[0], data.userIdAttribute);
    this.applyFieldValue(this.find('name', 'userAndGroupConfig.userRealNameAttribute')[0], data.userRealNameAttribute);
    this.applyFieldValue(this.find('name', 'userAndGroupConfig.emailAddressAttribute')[0], data.emailAddressAttribute);
    this.applyFieldValue(this.find('name', 'userAndGroupConfig.userPasswordAttribute')[0], data.userPasswordAttribute);
    this.applyFieldValue(this.find('name', 'userAndGroupConfig.groupBaseDn')[0], data.groupBaseDn);
    this.applyFieldValue(this.find('name', 'userAndGroupConfig.groupSubtree')[0], data.groupSubtree);
    this.applyFieldValue(this.find('name', 'userAndGroupConfig.groupObjectClass')[0], data.groupObjectClass);
    this.applyFieldValue(this.find('name', 'userAndGroupConfig.groupIdAttribute')[0], data.groupIdAttribute);
    this.applyFieldValue(this.find('name', 'userAndGroupConfig.groupMemberAttribute')[0], data.groupMemberAttribute);
    this.applyFieldValue(this.find('name', 'userAndGroupConfig.groupMemberFormat')[0], data.groupMemberFormat);
    this.applyFieldValue(this.find('name', 'userAndGroupConfig.userMemberOfAttribute')[0], data.userMemberOfAttribute);

    // special handling for memberOf attribute
    if (Ext.isEmpty(data.userMemberOfAttribute)) {
      this.find('name', 'groupType')[0].setValue('static');
      this.hideComponent(this.find('name', 'userAndGroupConfig.userMemberOfAttribute')[0]);
      this.showComponent(this.find('name', 'userAndGroupConfig.groupBaseDn')[0]);
      this.showComponent(this.find('name', 'userAndGroupConfig.groupSubtree')[0]);
      this.showComponent(this.find('name', 'userAndGroupConfig.groupObjectClass')[0]);
      this.showComponent(this.find('name', 'userAndGroupConfig.groupIdAttribute')[0]);
      this.showComponent(this.find('name', 'userAndGroupConfig.groupMemberAttribute')[0]);
      this.showComponent(this.find('name', 'userAndGroupConfig.groupMemberFormat')[0]);
    }
    else {
      this.find('name', 'groupType')[0].setValue('dynamic');
      this.showComponent(this.find('name', 'userAndGroupConfig.userMemberOfAttribute')[0]);
      this.hideComponent(this.find('name', 'userAndGroupConfig.groupBaseDn')[0]);
      this.hideComponent(this.find('name', 'userAndGroupConfig.groupSubtree')[0]);
      this.hideComponent(this.find('name', 'userAndGroupConfig.groupObjectClass')[0]);
      this.hideComponent(this.find('name', 'userAndGroupConfig.groupIdAttribute')[0]);
      this.hideComponent(this.find('name', 'userAndGroupConfig.groupMemberAttribute')[0]);
      this.hideComponent(this.find('name', 'userAndGroupConfig.groupMemberFormat')[0]);
    }

    // special handling for ldapGroupsAsRoles
    var ldapGroupsAsRoles = this.find('name', 'userAndGroupConfig.ldapGroupsAsRoles')[0];
    ldapGroupsAsRoles.setValue(data.ldapGroupsAsRoles);
    if (!data.ldapGroupsAsRoles) {
      ldapGroupsAsRoles.ownerCt.collapse();
    }

    var ldapGroupsAsRoles = this.find('name', 'userAndGroupConfig.ldapGroupsAsRoles')[0];
    if (!ldapGroupsAsRoles.getValue()) {
      ldapGroupsAsRoles.ownerCt.collapse();
    }

    var userPassAttrib = this.find('name', 'userAndGroupConfig.userPasswordAttribute')[0];
    if (Ext.isEmpty(userPassAttrib.getValue())) {
      userPassAttrib.ownerCt.collapse();
    }
    else {
      userPassAttrib.ownerCt.expand();
    }
  },
  hideComponent : function(component) {
    component.disable();
    component.hide();
  },
  showComponent : function(component) {
    component.enable();
    component.show();
  },
  handleButtons : function() {
    this.ldapConfigurationContainer.formPanel.buttons[0].hide();
    this.ldapConfigurationContainer.formPanel.buttons[1].show();
    this.ldapConfigurationContainer.formPanel.buttons[2].show();
  },
  checkUserMapping : function(formPanel, referenceData) {
    var ldapServerId, isNew = this.ldapConfigurationContainer.isNew,
    params = {
            userLimitCount : 20,
            _dc : new Date().getTime()
    };

    if (!isNew) {
      params.ldapServerId = this.ldapConfigurationContainer.record.get('id');
    }

    formPanel.getForm().doAction('sonatypeSubmit', {
      method : 'PUT',
      url : Sonatype.config.servicePath + '/ldap/test_user_conf',
      waitMsg : 'Checking user mapping...',
      fpanel : formPanel,
      params : params,
      dataModifiers: this.ldapConfigurationContainer.dataModifiers.submit,
      serviceDataObj : referenceData,
      autoValidation : false,
      success : function(form, action) {
        var window = new Sonatype.repoServer.LdapUserMappingPanel({});

        window.populateStore(Ext.decode(action.response.responseText));

        window.show();
      },
      failure : function(form, action) {
        Sonatype.utils.connectionError(action.response, 'Error checking user mapping.', false, {
          hideErrorStatus : true,
          decodeErrorResponse : true
        });
      }
    });
  },
  checkLogin : function(formPanel, referenceData) {
    var window = new Sonatype.repoServer.LdapLoginTestWindow({
      formPanel : formPanel,
      referenceData : referenceData,
      ldapConfigurationContainer : this.ldapConfigurationContainer
    });

    window.show();
  }
});