/*
 * Copyright (c) 2008-2013 Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global NX,Ext,Sonatype,Nexus*/
NX.define('Sonatype.repoServer.LdapUserMappingPanel', {
  extend : 'Ext.Window',
  constructor : function(cfg) {
    var config = cfg || {},
          defaultConfig = {
            title : 'User Mapping Test Results',
            layout : 'fit',
            height : 400,
            width : 600,
            autoScroll : true
          };

    Ext.apply(this, config, defaultConfig);

    this.userRecordConstructor = Ext.data.Record.create([
      {
        name : 'userId',
        sortType : Ext.data.SortTypes.asUCString
      },
      {
        name : 'name'
      },
      {
        name : 'email'
      },
      {
        name : 'roles'
      },
      {
        name : 'displayRoles',
        mapping : 'roles',
        convert : this.joinRoles.createDelegate(this)
      }
    ]);

    this.usersReader = new Ext.data.JsonReader({
      root : 'data',
      id : 'userId'
    }, this.userRecordConstructor);
    this.usersDataStore = new Ext.data.Store({
      reader : this.usersReader,
      sortInfo : {
        field : 'userId',
        direction : 'ASC'
      },
      autoLoad : false
    });

    this.usersGridPanel = new Ext.grid.GridPanel({
      frame : false,
      autoScroll : true,
      ds : this.usersDataStore,
      sortInfo : {
        field : 'userId',
        direction : 'asc'
      },
      loadMask : true,
      height : 400,
      width : 650,
      deferredRender : false,
      columns : [
        {
          header : 'User ID',
          dataIndex : 'userId',
          width : 100,
          id : 'user-config-userid-col'
        },
        {
          header : 'Name',
          dataIndex : 'name',
          width : 175,
          id : 'user-config-name-col'
        },
        {
          header : 'Email',
          dataIndex : 'email',
          width : 175,
          id : 'user-config-email-col'
        },
        {
          header : 'Roles',
          dataIndex : 'displayRoles',
          width : 175,
          id : 'user-config-roles-col'
        }
      ],
      disableSelection : true,
      viewConfig : {
        emptyText : 'No user records found.'
      }
    });

    Sonatype.repoServer.LdapUserMappingPanel.superclass.constructor.call(this, {
      modal : true,
      items : [this.usersGridPanel],
      buttonAlign : 'center',
      buttons : [
        {
          text : 'Close',
          handler : function() {
            this.close();
          },
          scope : this
        }
      ]
    });
  },

  populateStore : function(data) {
    this.usersDataStore.loadData(data ? data : {
      data : []
    });
  },
  joinRoles : function(value, parent) {
    var s = "";
    for (var i = 0; i < value.length; i++) {
      if (i > 0) {
        s += ', ';
      }
      s += value[i];
    }
    return s;
  }
});