/*
 * Copyright (c) 2008-2013 Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
define('Sonatype/repoServer/M2SettingsEditor', function() {

  Sonatype.repoServer.M2SettingsEditor = function(config) {
    var config = config || {};
    var defaultConfig = {
      uri : Sonatype.config.servicePath + '/templates/settings',
      resetButton : true,
      defaultTimeoutValues : {},
      dataModifiers : {
        load : {},
        save : {}
      },
      referenceData : {
        id : "",
        settingsXmlText : ""
      }
    };
    Ext.apply(this, config, defaultConfig);

    this.checkPayload();

    Sonatype.repoServer.M2SettingsEditor.superclass.constructor.call(this, {
          listeners : {
            submit : {
              fn : this.submitHandler,
              scope : this
            }
          },
          autoScroll : false,
          items : [{
                xtype : 'textfield',
                fieldLabel : 'Template ID',
                itemCls : 'required-field',
                helpText : 'M2 Settings Template Id',
                name : 'id',
                width : 300,
                allowBlank : false,
                readOnly : !this.isNew,
                validator : Sonatype.utils.validateId
              }, {
                xtype : 'textfield',
                fieldLabel : 'Settings Template',
                helpText : 'Enter Maven 2 settings.xml content here.<br/>Substitution keywords:<br/>  Base URL: ${baseurl}<br/>  Current user&#39;s id: ${userId}',
                hidden : true,
                itemCls : 'required-field'
              }, {
                // we are using the textfield above for the lable,
                // this does 2 things, it puts the help text above the field
                // and make the width of the text area ~100%
                xtype : 'textarea',
                itemCls : 'required-field',
                name : 'settingsXmlText',
                anchor : Sonatype.view.FIELD_OFFSET + ' 75%',
                allowBlank : false,
                readOnly : this.payload.data.readonly,
                hideLabel : true
              }]
        });
  };

  Ext.extend(Sonatype.repoServer.M2SettingsEditor, Sonatype.ext.FormPanel, {
        submitHandler : function(form, action, receivedData) {
          if (this.isNew)
          {
            return;
          }

          var rec = this.payload;
          rec.beginEdit();
          rec.set('id', receivedData.id);
          rec.set('settingsXmlText', receivedData.settingsXmlText);
          rec.commit();
          rec.endEdit();
        },

        loadData : function() {
          if (this.isNew)
          {
            var templateModifiers = Ext.apply({}, {
                  id : Sonatype.utils.returnEmptyStr
                }, this.dataModifiers.load);

            this.form.on('actioncomplete', function(form, action) {
                  form.clearInvalid();
                }, {
                  single : true
                });
            this.form.doAction('sonatypeLoad', {
                  url : Sonatype.config.servicePath + '/templates/settings/default',
                  method : 'GET',
                  fpanel : this,
                  dataModifiers : templateModifiers,
                  scope : this
                });
          }
          else
          {
            Sonatype.repoServer.M2SettingsEditor.superclass.loadData.call(this);
          }

          // disable the save button if the template is readonly
          this.buttons[0].setDisabled(this.payload.data.readonly);

          this.doLayout();
        }

      });

  Sonatype.Events.addListener('templateViewInit', function(cardPanel, rec, gridPanel) {
        var sp = Sonatype.lib.Permissions;
        if (sp.checkPermission('nexus:settingstemplates', sp.READ))
        {

          cardPanel.add(new Sonatype.repoServer.M2SettingsEditor({
                tabTitle : 'Configuration',
                name : 'configuration',
                payload : rec
              }));
        }
      });

});