/*
 * Copyright (c) 2008-2013 Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global Ext, Sonatype*/

define('Sonatype/repoServer/StagingRuleSet', function() {

    Sonatype.repoServer.StagingRuleSet = function(cfg) {
      var
            config = cfg || {},
            defaultConfig = {
              title : 'Staging Ruleset'
            },
            sp = Sonatype.lib.Permissions,
            toolbar = [];

      Ext.apply(this, config, defaultConfig);

      Sonatype.repoServer.StagingRuleSet.superclass.constructor.call(this, {
            url : Sonatype.config.servicePath + '/staging/rule_sets',
            tbar : toolbar,
            dataAutoLoad : false,
            tabbedChildren : true,
            addMenuInitEvent : 'rulesetAddMenuInit',
            deleteButton : sp.checkPermission('nexus:stagingrulesets', sp.DELETE),
            rowClickEvent : 'rulesetViewInit',
            dataSortInfo : {
              field : 'name',
              direction : 'asc'
            },
            columns : [{
                  name : 'resourceURI'
                }, {
                  name : 'id'
                }, {
                  name : 'name',
                  header : 'Name',
                  width : 100
                }, {
                  name : 'description',
                  header : 'Description',
                  width : 200
                }, {
                  name : 'rules'
                }]
          });

      this.dataStore.load();
    };

    Ext.extend(Sonatype.repoServer.StagingRuleSet, Sonatype.panels.GridViewer, {
      emptyText: 'No rulesets defined',
      emptyTextWhileFiltering: 'No rulesets matched criteria: {criteria}'
    });

    Sonatype.Events.addListener('nexusNavigationInit', function(nexusPanel) {
          nexusPanel.add({
                enabled : Sonatype.lib.Permissions.checkPermission('nexus:stagingrulesets', Sonatype.lib.Permissions.READ),
                sectionId : 'st-enterprise-staging',
                title : 'Staging Ruleset',
                tabId : 'staging-ruleset',
                tabCode : Sonatype.repoServer.StagingRuleSet
              });
        });

    Sonatype.Events.addListener('rulesetAddMenuInit', function(menu) {
      var sp = Sonatype.lib.Permissions;

      if (sp.checkPermission('nexus:stagingrulesets', sp.CREATE)) {
        menu.add(['-', {
          text : 'Ruleset',
          autoCreateNewRecord : true,
          handler : function(container, rec, item, e) {
            rec.beginEdit();
            rec.set('resourceURI', 'new');
            rec.set('rules', []);
            rec.commit();
            rec.endEdit();
          }
        }]);
      }
    });

});
